#!/usr/bin/perl -w
###############################################################################
#    Copyright (C) 2002-2204 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: afickonfig.pl,v 1.7 2004/10/18 08:23:17 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
#                         perl modules
###############################################################################

use strict;
use diagnostics;

use Getopt::Long;
use Carp qw(cluck);

###############################################################################
#                     global variables
###############################################################################

my $Version = '0.2-0';

# connection between command line options and config parameters

#######################################################
# usage
# print some help
sub usage($) {
	my $version = shift(@_);
	print <<EOHELP;

Usage: $0 [afickonfig option] [afick options]

afickonfig options (control afickonfig)  : 
 -c|--config_file file        configname of config file to use
 -h|--help                    show this help page
 --print_config               display internals variables after arguments and config file parsing
 				(for debugging purposes)
 -V|--version                 show afickonfig version

afick options (change afick's options) :

 -a|--ignore_case             helpful on windows plateforms, dangerous on unix ones
 				reverse : --noignore_case
 -d|--debug level	      set a level of debugging messages, from 0 (none) to 3 (full)
 				default : 0
 -D| --database file          force the database name 
 -f|--full_newdel             report full information for new or deleted directories
                               default: no
			       reverse : --nofull_newdel 
 -m|--missing_files           warn about files declared in config files 
                               which do not exists, 
			       default: no; 
			       reverse : --nomissing_files
 -r|--running_files           warn about "running" files : modified since program begin
                               default: no 
                               reverse: --norunning_files
 -s|--dead_symlinks           warn about dead symlinks 
                               default: no 
                               reverse: --nodead_symlinks
 -t|--timing		      Print timing statistics
				default: no
				reverse : --notiming
 -x|--exclude_suffix ext1 ext2        list of suffixes to ignore
 -X|--exclude_prefix ext1 ext2        list of prefix to ignore
 -y|--history file	      history file of all runs with summary
 -A|--archive directory	      directory where archive files are stored

Disclaimer:
This script is intended to provide a means for
detecting changes made to files, via a regular
comparison of MD5 hashes to an established baseline. 

Copyright (c) 2002 Eric Gerbier <gerbier\@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
EOHELP

}

#############################################################
sub print_config ($) {
	my $configfile = shift(@_);
	system("afick -c $configfile --print_config");
}
#############################################################
sub change_config($%) {
	my $configfile = shift(@_);
	my %changes    = @_;

	open( CONFIG, '<', $configfile ) or die "can not open $configfile : $!\n";
	my @config = <CONFIG>;
	close(CONFIG);

	my $nb_changes = 0;
	foreach my $key ( keys %changes ) {
		my $found     = 0;
		my @newconfig = ();
		foreach my $ligne (@config) {
			if ( $ligne =~ m/^\s*$key\s*:=\s*(.*)\s*$/ ) {
				my $val = $1;
				if ( $val eq $changes{$key} ) {
					print "no changes for $key\n";
				}
				else {
					print "replace $ligne by $key := $changes{$key}\n";
					$ligne = "$key := $changes{$key}\n";
					$nb_changes++;
				}
				$found = 1;
			}
			push( @newconfig, $ligne );
		}
		if ( !$found ) {
			print "add line $key := $changes{$key}\n";
			push( @newconfig, "$key := $changes{$key}\n" );
			$nb_changes++;
		}
		@config = @newconfig;
	}
	if ($nb_changes) {

		# save modified config file
		print "rewrite changed $configfile ($nb_changes)\n";
		open( CONFIG, '>', $configfile )
		  or die "can not write to $configfile : $!\n";
		foreach my $ligne (@config) {
			print CONFIG $ligne;
		}
		close(CONFIG);
	}
	return $nb_changes;
}
#############################################################
sub version($) {
	my $version = shift(@_);
	print "\n";
	print
"afickonfig : another file integrity checker configurator\nversion $version\n";
}
#############################################################
#                          main
#############################################################

my $default_config_file;
if ( $^O =~ m/linux/ ) {
	$default_config_file = '/etc/afick.conf';
}
elsif ( $^O =~ m/^MSWin/ ) {
	$default_config_file = 'windows.conf';

}
else {
	$default_config_file = 'afick.conf';
}

$| = 1;

# variables for parameter analysis
my $configfile;    # config file name
my $help;
my $version;
my $print_config;

# parameters for afick
my (
	$Archive,   $Debug_level,       $Ignore_case, $Report_full_newdel,
	$History,   $Warn_missing_file, $Running,     $Warn_dead_symlinks,
	$sufx_list, $prefx_list,        $Timing,      $Database
);

Getopt::Long::Configure('no_ignore_case');
unless (
	GetOptions(

		# afickonfig options
		'config_file|c=s' => \$configfile,
		'help|h'          => \$help,
		'print_config'    => \$print_config,
		'version|V'       => \$version,

		# afick options
		'archive=s'          => \$Archive,
		'database|D=s'       => \$Database,
		'ignore_case|a!'     => \$Ignore_case,
		'full_newdel|f!'     => \$Report_full_newdel,
		'history|y=s'        => \$History,
		'missing_files|m!'   => \$Warn_missing_file,
		'running_files|r!'   => \$Running,
		'dead_symlinks|s!'   => \$Warn_dead_symlinks,
		'exclude_suffix|x=s' => \$sufx_list,
		'exclude_prefix|X=s' => \$prefx_list,
		'timing|t!'          => \$Timing,
		'debug|d=i'          => \$Debug_level

	)
  )
{
	usage($Version);
	die "abort : incorrect option\n";
}

if ($help) {

	# -h : help
	usage($Version);
	exit;
}
elsif ($version) {

	# -V : version
	version($Version);
	exit;
}

if ($configfile) {
}
elsif ( -e $default_config_file ) {
	$configfile = $default_config_file;
}
else {
	usage($Version);
	die
"abort : missing configfile name (-c flag) and default config file $default_config_file\n";
}

# some more controls
if ( !-e $configfile ) {
	die "abort : missing configfile name $configfile\n";
}
elsif ( !-w $configfile ) {
	die "abort : configfile name $configfile is not writable\n";
}

if ($print_config) {
	print_config($configfile);
	exit;
}

my %changes;
$changes{'archive'}     = $Archive     if ( defined $Archive );
$changes{'database'}    = $Database    if ( defined $Database );
$changes{'debug'}       = $Debug_level if ( defined $Debug_level );
$changes{'history'}     = $History     if ( defined $History );
$changes{'ignore_case'} = $Ignore_case if ( defined $Ignore_case );
$changes{'report_full_newdel'} = $Report_full_newdel
  if ( defined $Report_full_newdel );
$changes{'running_files'} = $Running if ( defined $Running );
$changes{'warn_dead_symlinks'} = $Warn_dead_symlinks
  if ( defined $Warn_dead_symlinks );
$changes{'warn_missing_file'} = $Warn_missing_file
  if ( defined $Warn_missing_file );
$changes{'exclude_suffix'} = $sufx_list  if ( defined $sufx_list );
$changes{'exclude_prefix'} = $prefx_list if ( defined $prefx_list );
$changes{'timing'}         = $Timing     if ( defined $Timing );

my $return_value = change_config( $configfile, %changes );

exit $return_value;

__END__


=head1 NAME

afickonfig - a tool to manage Afick's config files

=head1 DESCRIPTION

C<afickonfig> is to change parameters in afick's config file in a batch way. It was designed to work with same options names as afick.
It only work on global config parameters, not on file directives.

The idea came from the "postconf" utility from postfix.

=head1 SYNOPSIS

afickonfig  [L<options|options>] [L<action|actions>] 

afick use posix syntaxe, which allow many possibilities : 

=over 4

=item *

long (--) options

=item *

short (-) options

=item *

negative (--no) options

=back

=head1 OPTIONS

options are used to control afickconfig

=over 4

=item *
--config_file|-c configfile

read the configuration in config file named "configfile".

=item *
--help|-h

Output help information and exit.

=item *
--print_config

display internals variables after arguments and config file parsing (for debugging purposes)

=item *
--version|-V

Output version information and exit.

=back

=head1 ACTIONS

actions are used to change afick's configuration

=over 4

=item *
--archive|-A directory

write reports to "directory".

=item *
--database|-D name

use the database named "name".

=item *
--debug|-d level

set a level of debugging messages, from 0 (none) to 3 (full)

=item *
--full_newdel|-f,(--nofull_newdel)

(do not) report full information on new and deleted directories. Default : no

=item *
--history|-y historyfile

write session status to history file

=item *
--ignore_case|-a

ignore case for file names. Can be helpfull on windows plateforms, but is dangerous on unix ones.

=item *
--missing_files|-m,(--nomissing_files)

(do not) warn about files declared in config files which does not exists. Default : no

=item *
--dead_symlinks|-s,(--nodead_symlinks)

(do not) warn about dead symlinks. Default : no

=item *
--running_files|-r,(--norunning_files)

(do not) warn about "running" files : modified since program begin. Default : no

=item *
--timing|-t,(--notiming)

(do not) Print timing statistics. Default : no

=item *
--exclude_suffix|-x "ext1 ext2 ... extn"

list of suffixes to ignore

=item *
--exclude_prefix|-X "ext1 ext2 ... extn"

list of prefix to ignore

=back

=head1 FILES

if no config file on command line, afick try to open F</etc/afick.conf> (unix) or F<windows.conf> (windows) as
default config

for config file syntax see afick.conf(5)

=head1 USE

To use this program, you must just run it with same afick command line options :

C<afickonfig -c linux.conf --timing --norunnig_files --debug=1 --archive=afick_archive>

=head1 RETURN VALUES

The exit status is the number of real changes

=head1 NOTES

this program only use perl and its standard modules.

=head1 SEE ALSO

=for html
<a href="afick.conf.5.html">afick.conf(5)</a> for configuration file
<br>
<a href="afick-tk.1.html">afick-tk(1)</a> for graphical interface
<br>
<a href="afick.1.html">afick(1)</a> for the command-line interface

=for man
\fIafick.conf\fR\|(5) for configuration file
.PP
\fIafick\-tk\fR\|(1) for graphical interface
.PP
\fIafick\fR\|(1) for the command-line interface

=head1 COPYRIGHT

Copyright (c) 2002,2003,2004 Eric Gerbier
All rights reserved.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

=head1 AUTHORS

Eric Gerbier

you can report any bug or suggest to gerbier@users.sourceforge.net
